# autogenerated
from typing import Literal

LayerTitle = Literal[
    "Водная эрозия",
    "Линейная эрозия",
    "Ветровая эрозия",
    "Опустынивание",
    "Переувлажнение",
    "Подтопление",
    "Заболачивание",
    "Затопление",
    "Захламление",
    "Обвально-осыпные и оползневые процессы",
    "Абразия",
    "Кадастровые округа",
    "Территории объектов культурного наследия",
    "Иные нарушенные земли ",
    "Кадастровые районы ",
    "Нарушенные земли при наземном строительстве ",
    "Кадастровые кварталы",
    "Нарушенные земли при гидротехническом строительстве",
    "Нарушенные земли при недропользовании",
    "Нарушенные земли при промышленном лесопользовании",
    "Нарушенные земли при сельскохозяйственном освоении",
    "Нарушенные земли при проведении геологоразведочных, испытательных, эксплуатационных и иных работ",
    "Нарушенные земли при складировании и захоронении промышленных отходов, загрязнение земель",
    "Государственная граница Российской Федерации",
    "Гари",
    "Негативный процесс отсутствует",
    "Субъекты Российской Федерации (линии)",
    "Субъекты Российской Федерации (полигоны)",
    "Засоление",
    "Муниципальные образования (полигональный)",
    "Муниципальные образования (линейный)",
    "Населённые пункты (полигоны)",
    "Населённые пункты (линии)",
    "Земельные участки из ЕГРН",
    "Земельные участки, образуемые по схеме расположения земельного участка",
    "Земельные участки, выставленные на аукцион ",
    "Земельные участки, свободные от прав третьих лиц",
    "Земельные участки, образуемые по проекту межевания территории",
    "Здания",
    "Сооружения",
    "Объекты незавершенного строительства",
    "Единые недвижимые комплексы",
    "Предприятие как имущественный комплекс",
    "ЗОУИТ объектов культурного наследия",
    "ЗОУИТ объектов энергетики, связи, транспорта",
    "ЗОУИТ природных территорий",
    "ЗОУИТ охраняемых объектов и безопасности",
    "Иные ЗОУИТ",
    "Территориальные зоны",
    "Красные линии ",
    "Особо охраняемые природные территории ",
    "Охотничьи угодья ",
    "Лесничества",
    "Граница лесопарка",
    "Береговые линии (границы водных объектов) (полигональный)",
    "Береговые линии (границы водных объектов)(линейный)",
    "Особые экономические зоны",
    "Территории опережающего развития",
    "Игорные зоны",
    "Территории выполнения комплексных кадастровых работ",
    "Территория проведения мероприятий по ликвидации накопленного вреда окружающей среде, образовавшегося в результате производства химической продукции в г. Усолье-Сибирское Иркутской области",
    "Кадастровая стоимость объекта",
    "Удельный показатель кадастровой стоимости",
]
